use std::path::Path;

use s2n_quic::Server;
use tokio::io::AsyncWriteExt;

use crate::ServerConfig;

pub async fn run<'a>(server_config: ServerConfig<'a>) {
    let mut server = Server::builder()
        .with_io(server_config.server_address)
        .unwrap()
        .with_tls((
            Path::new("certificates/cert.pem"),
            Path::new("certificates/key.pem"),
        ))
        .unwrap()
        .start()
        .unwrap();
    while let Some(mut connection) = server.accept().await {
        println!(
            "Server Name = {}",
            connection.server_name().unwrap().unwrap().to_string()
        );

        println!(
            "Client Address = {}",
            connection.remote_addr().unwrap().to_string()
        );
        while let Ok(Some(mut stream)) = connection.accept_bidirectional_stream().await {
            println!(
                "{}",
                String::from_utf8(stream.receive().await.unwrap().unwrap().to_vec()).unwrap()
            );
            match stream.send("Hi from Tahinli too".as_bytes().into()).await {
                Ok(_) => {
                    if let Err(err_val) = stream.flush().await {
                        eprintln!("Error: Flush | {}", err_val);
                    }
                }
                Err(err_val) => eprintln!("Error: Send | {}", err_val),
            }
            if let Err(err_val) = stream.close().await {
                eprintln!("Error: Stream Close | {}", err_val);
            }

            if let Err(err_val) = stream.shutdown().await {
                eprintln!("Error: Stream Shutdown | {}", err_val);
            }
            return;
        }
        return;
    }
}
